@@ -38,6 +38,9 @@ class DigitalPass(GObject.GObject):
     def barcodes(self):
         raise NotImplementedError()
 
+    def creator(self):
+        raise NotImplementedError()
+
     def description(self):
         raise NotImplementedError()
 
@@ -67,6 +70,9 @@ class DigitalPass(GObject.GObject):
     def mime_type():
         raise NotImplementedError()
 
+    def relevant_date():
+        raise NotImplementedError()
+
     def set_path(self, new_path: str):
         self.__path = new_path
 
@@ -89,7 +95,14 @@ class Barcode:
 
     def __init__(self, barcode_dictionary):
         self.__format = barcode_dictionary['format']
+        if not self.__format:
+            # Format is a required field
+            raise Exception()
+
         self.__message = barcode_dictionary['message']
+        if not self.__message:
+            # Message is a required field
+            raise Exception()
 
         self.__message_encoding = None
         if 'messageEncoding' in barcode_dictionary.keys():
@@ -282,6 +295,33 @@ class Date:
 
     @classmethod
     def from_iso_string(cls, string):
+        """
+        Creates a Date corresponding to the given ISO 8601 formatted string
+        """
+
+        # Include hours and seconds if they have not been specified.
+        # Why? DateTime.new_from_iso8601 does not accept times without them.
+
+        matches = re.finditer('(T|t)([0-9]{2}\:?)+(\+|\-|Z)?', string)
+
+        for match in matches:
+            missing_info = None
+            colon_count = match.group(0).count(':')
+
+            if colon_count == 0:
+                # Minutes and seconds are missing
+                missing_info = ':00:00'
+            elif colon_count == 1:
+                # Seconds are missing
+                missing_info = ':00'
+
+            if missing_info:
+                insertion_index = match.end() - 1
+                string = string[:insertion_index] + missing_info + string[insertion_index:]
+
+            # We are only interested in the first match
+            break
+
         date = GLib.DateTime.new_from_iso8601(string)
         return Date(date)
 
@@ -333,6 +373,9 @@ class PassDataExtractor:
 
             if key in self._dictionary:
                 value = self._dictionary[key]
+            else:
+                # The key does not exist... nothing to do
+                return None
 
             if not type_constructor and type(value) == dict:
                 return PassDataExtractor(value)
@@ -370,10 +413,13 @@ class PassDataExtractor:
 
         result = list()
         for item_data in data_list:
-            arguments = (item_data,) + extra_arguments
 
             if item_constructor:
-                instance = item_constructor(*arguments)
+                try:
+                    arguments = (item_data,) + extra_arguments
+                    instance = item_constructor(*arguments)
+                except:
+                    continue
             else:
                 instance = item_data
 
