@@ -83,7 +83,7 @@ class EsPass():
     # Metadata
 
     def creator(self):
-        return self.__data.get('creator')
+        return self.__data.get('creator') or _('Unknown')
 
 
     # Fields
@@ -124,6 +124,9 @@ class EsPassAdapter(DigitalPass):
     def barcodes(self):
         return [self.__adaptee.barcode()]
 
+    def creator(self):
+        return self.__adaptee.creator()
+
     def description(self):
         return self.__adaptee.description()
 
@@ -153,9 +156,11 @@ class EsPassAdapter(DigitalPass):
     def mime_type():
         return 'application/vnd.espass-espass+zip'
 
+    def relevant_date(self):
+        return None
+
     def unique_identifier(self):
-        return '.'.join([self.format(),
-                         self.__adaptee.id()])
+        return '.'.join([self.__adaptee.id(), self.format()])
 
     def voided(self):
         return False
