@@ -15,8 +15,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Adw, Gtk
+from gi.repository import Adw, GObject, Gtk
 
+from .digital_pass import DigitalPass
+from .digital_pass_list_store import SortingCriteria
 from .pass_row import PassRow
 
 
@@ -25,11 +27,18 @@ class PassList(Gtk.ListBox):
 
     __gtype_name__ = 'PassList'
 
+    __gsignals__ = {
+        'pass-activated' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (DigitalPass,)),
+        'pass-selected' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, (DigitalPass,))
+    }
+
     def __init__(self):
         super().__init__()
 
+        self.__list_model = None
         self.__selected_row = None
-        self.set_header_func(self.on_update_header)
+        self.__sorting_criteria = None
+        self.set_header_func(self._on_update_headers)
 
         # Create a placeholder widget to be displayed when the list is empty
         placeholder = Adw.StatusPage.new()
@@ -38,36 +47,42 @@ class PassList(Gtk.ListBox):
         placeholder.set_description(_('Use the “+” button to import a pass'))
         self.set_placeholder(placeholder)
 
-        self.connect('row-activated', self.on_row_activated)
+        self.connect('row-activated', self._on_row_activated)
 
-    def bind_model(self, pass_list_model):
-        super().bind_model(pass_list_model.get_model(), PassRow)
-
-    def on_row_activated(self, pass_list, pass_row):
+    def _on_row_activated(self, pass_list, pass_row):
         self.__selected_row = pass_row
+        self.emit('pass-activated', pass_row.data())
 
-    def on_update_header(self, row, row_above):
-        row_date = row.data().expiration_date()
-        row_header = row_date.as_relative_pretty_string() if row_date else None
+    def _on_update_headers(self, row, row_above):
+        row.update_header_text_for(self.__sorting_criteria)
 
-        row_above_date = row_above.data().expiration_date() if row_above else None
-        row_above_header = row_above_date.as_relative_pretty_string()\
-            if row_above and row_above_date else None
+        if row_above:
+            row_above.update_header_text_for(self.__sorting_criteria)
 
-        if not row_above or row_header != row_above_header:
+        if not row_above or row.header_text() != row_above.header_text():
             row.show_header()
         else:
             row.hide_header()
 
+    def bind_model(self, pass_list_model):
+        self.__sorting_criteria = pass_list_model.sorting_criteria()
+        self.__list_model = pass_list_model
+        super().bind_model(pass_list_model.get_model(), PassRow)
+
+    def row_at_index(self, index):
+        row_at_index = self.get_row_at_index(index)
+
+        if not row_at_index:
+            row_at_index = self.get_row_at_index(0)
+
+        return row_at_index if row_at_index else None
+
     def select_pass_at_index(self, index):
-        selected_row = self.get_row_at_index(index)
+        row_at_index = self.row_at_index(index)
 
-        if not selected_row:
-            selected_row = self.get_row_at_index(0)
-
-        if selected_row:
-            self.select_row(selected_row)
-            self.emit('row-activated', selected_row)
+        if row_at_index:
+            self.select_row(row_at_index)
+            self.emit('pass-selected', row_at_index.data())
 
     def selected_pass(self):
         selected_pass = None
@@ -85,6 +100,7 @@ class PassList(Gtk.ListBox):
 
         return index
 
+    def sort_by(self, sorting_criteria):
+        self.__sorting_criteria = sorting_criteria
+        self.__list_model.sort_by(sorting_criteria)
 
-
-    
